@@ -46,7 +46,7 @@ public class DebuggingClassWriter extends ClassVisitor {
     }
     
     public DebuggingClassWriter(int flags) {
-	super(Opcodes.ASM6, new ClassWriter(flags));
+	super(Constants.ASM_API, new ClassWriter(flags));
     }
 
     public void visit(int version,
