@@ -554,7 +554,7 @@ public class Enhancer extends AbstractClassGenerator
         }
         CollectionUtils.filter(methods, new RejectModifierPredicate(Constants.ACC_STATIC));
         CollectionUtils.filter(methods, new VisibilityPredicate(superclass, true));
-        CollectionUtils.filter(methods, new DuplicatesPredicate());
+        CollectionUtils.filter(methods, new DuplicatesPredicate(methods));
         CollectionUtils.filter(methods, new RejectModifierPredicate(Constants.ACC_FINAL));
     }
 
